unit UEditor;
//*****************************************************************************
// Unit UEditor         - Diese Formular-Unit enthlt alle Routinen zum
// -------------          Anzeigen, Bearbeiten, Hinzufgen und Lschen der
//                        Spiele-Grafiken im Editor
//
// ffentliche-Funktionen:
// -----------------------
// SetGamePath          - Setzt den zu verwendenen Pfad des Spiels
//
// Verwendet eigene Units:
// -----------------------
// UType                - Enthlt wichtige Typen
// UExtImgLst           - Enthlt Routinen fr eine Bilderliste
// UFile                - enthlt Routinen zum Lesen der Spiele-Grafiken aus
//                        verschiedenen Dateien
//
// Vordiploms-Programm  - (c) Januar 2003,  by Andreas Schibilla
// -------------------
//
//*****************************************************************************
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ColorGrd, StdCtrls, Spin, Grids, UType, UExtImgLst,
  ExtCtrls;

type
  TEditorForm = class(TForm)
    ColorGrid: TColorGrid;
    BtnCancel: TButton;
    BtnSave: TButton;
    GrBitmaps: TGroupBox;
    LaLevel: TLabel;
    SpEdBmpNum: TSpinEdit;
    BtnAdd: TButton;
    BtnDelete: TButton;
    EditorGrid: TDrawGrid;
    GrAnzeige: TGroupBox;
    SpEdWidth: TSpinEdit;
    SpEdHeight: TSpinEdit;
    LaWidth: TLabel;
    LaHeight: TLabel;
    LaSize: TLabel;
    SpEdSize: TSpinEdit;
    BtnImport: TButton;
    LaSizeInfo: TLabel;
    LaZoom: TLabel;
    SpEdZoom: TSpinEdit;
    ChLines: TCheckBox;
    BtnLoad: TButton;
    BtnExport: TButton;
    OpenDialog: TOpenDialog;
    PaNum: TPanel;
    SaveDialog: TSaveDialog;
    LoadDialog: TOpenDialog;
    procedure EditorGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure BtnSaveClick(Sender: TObject);
    procedure GridMustChange(Sender: TObject);
    procedure BtnImportClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EditorGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure ChLinesClick(Sender: TObject);
    procedure BtnExportClick(Sender: TObject);
    procedure SpEdZoomChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BmpChange(Sender: TObject);
    procedure BtnLoadClick(Sender: TObject);
    procedure BtnAddClick(Sender: TObject);
    procedure BtnDeleteClick(Sender: TObject);
  private
    { Private-Deklarationen }
    g_GameBmp   : TExtImgLst;
    g_BufferBmp : TBitmap;
    CurBmpNum   : Word;
    g_GamePath  : String;
    procedure SetBmpPanelName(Num : Word);
    procedure LoadBmpToGrid(Bmp : TBitmap; Size : Byte);
    procedure SaveCurImgLst(FileName : TFileName);
    procedure ImportBitmap(FileName : TFileName);
    procedure DeleteEnemy(Num : Word);
    procedure AddEnemy;
    procedure OpenGameBitmaps(FileName : TFileName);
  public
    { Public-Deklarationen }
    procedure SetGamePath(Path : String);
  end;

var
  EditorForm: TEditorForm;

implementation

uses UFile;

{$R *.dfm}


//*****************************************************************************
// Setzt den Namen des aktuell ausgewhlten Bildes
//
//  ->   Num         Welche Bild-Nummer hat das aktuelle Bild
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.SetBmpPanelName(Num : Word);
begin
 case Num of                                                // welche Nummer?
   1: PaNum.Caption := 'Spieler Raumschiff';                // setze richtigen Namen
   2: PaNum.Caption := 'Spieler Schu';
   3: PaNum.Caption := 'Gegner Schu';
 else
      PaNum.Caption := 'Gegner Nr. '+IntToStr(Num-3);       // Gegner-Raumschiffe durchnummerieren
 end;
end;


//*****************************************************************************
// Zeichnet ein Bitmap zum Editieren in das Grid-Feld
//
//  ->   Bmp         Bitmap, das angezeigt und bearbeitet werden soll
//  ->   Size        Gre (in %) des Bitmaps auf dem Spielfeld
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.LoadBmpToGrid(Bmp : TBitmap; Size : Byte);
begin
 g_BufferBmp.Assign(Bmp);                                   // setze Buffer
 EditorGrid.DefaultColWidth  :=  SpEdZoom.Value;            // stelle Grid-Gre passend zur Zoom-Stellung
 EditorGrid.DefaultRowHeight :=  SpEdZoom.Value;
 EditorGrid.RowCount         :=  g_BufferBmp.Height;        // setze neue Hhe und Breite
 EditorGrid.ColCount         :=  g_BufferBmp.Width;

 EditorGrid.GridLineWidth := Ord(ChLines.Checked);          // sollen Linien zwischen den Kstchen sein?

 SpEdWidth.Tag    := 1;                                     // kleiner Trick:
 SpEdHeight.Tag   := 1;                                     // um hier das autom. Aufrufen von OnChange zu verhindern
 SpEdWidth.Value  := g_BufferBmp.Width;                     // Dann werden Breite und Hhe neu gesetzt
 SpEdHeight.Value := g_BufferBmp.Height;
 SpEdSize.Value   := Size;                                  // und die Gre (in %) wird auch aktualisiert
 SpEdWidth.Tag    := 0;
 SpEdHeight.Tag   := 0;

 SetBmpPanelName(SpEdBmpNum.Value);                         // Setze Namen fr das Bild
 EditorGrid.Repaint;                                        // Aktualisiere das Grid-Feld
end;


//*****************************************************************************
// Entfernt einen Gegner aus der Bilderliste
//
//  ->   Num         Numer des zu lschenden Gegners
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.DeleteEnemy(Num : Word);
begin
 if (Num > 3) AND (g_GameBmp.Count > 4) then                // > 3 = Gegner; > 4 = mind. zwei Gegner vorhanden
 begin
    SpEdBmpNum.Value := 1;                                  // zeige wieder Player an
    g_GameBmp.DelBmp(Num);                                  // entferne das Bild des Gegners
    SpEdBmpNum.MaxValue := g_GameBmp.Count;                 // keinen ungltigen Bereich zulassen
 end;
end;


//*****************************************************************************
// Speichert die aktuellen Spiele-Grafiken in einer Datei ab!
//
//  ->   FileName    Name der Datei, in die gespeichert wird
//
// Wird fr jeden zu zeichnenden Kasten aufgerufen (Owner-Drawn-Control)
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.SaveCurImgLst(FileName : TFileName);
begin
 g_GameBmp.ChangeBmp(CurBmpNum, g_BufferBmp, SpEdSize.Value);           // Speicher das aktuelle Bild
 if not SaveGameBitmaps(g_GameBmp, FileName) then                       // und schreibe die Bilderliste in eine Datei
    ShowMessage(C_ERR_SAVE_BITMAPS+Filename);                           // Bei Fehler Meldung ausgeben
end;


//*****************************************************************************
// Bietet die Mglichkeit ein Bitmap als Spielegrafik zu Importieren
//
//  ->   FileName    Name des zu importierenden Bitmaps
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.ImportBitmap(FileName : TFileName);
var
 Bmp : TBitmap;
begin
 try
    Bmp := TBitmap.Create;                                  // lege neues Bild an
    Bmp.LoadFromFile(FileName);                             // ffne es aus der Datei
    g_BufferBmp.Assign(Bmp);                                // setze es aktuell in Buffer
    LoadBmpToGrid(Bmp, 12);                                 // und zeige es an
 except
    on Exception do                                         // Fange Fehler beim Laden z.B. falsches Format ab
       ShowMessage(C_ERR_IMPORT_BITMAP+FileName);           // und zeige Fehlermeldung
 end;
 Bmp.Free;                                                  // gebe Temp-Bitmap wieder frei
end;


//*****************************************************************************
// Fgt ein leeres Bild ans Ender der Bilder-Liste an
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.AddEnemy;
var
 tmpBmp : TBitmap;
begin
 tmpBmp := TBitmap.Create;                            // erzeuge Standard-Bild
 tmpBmp.Width  := 10;
 tmpBmp.Height := 10;
 tmpBmp.Canvas.Brush.Color := clBlack;                // Flle Standard Bild schwarz
 tmpBmp.Canvas.FillRect(tmpBmp.Canvas.ClipRect);
 g_GameBmp.AddBmp(tmpBmp,5);                          // und fge Bild hinten an die Liste (5 = Size)
 SpEdBmpNum.MaxValue := g_GameBmp.Count;
 SpEdBmpNum.Value    := g_GameBmp.Count;              // SpEd-nderung -> Das neue Bild wird
 tmpBmp.Free;                                         // wegen OnChange autom. angezeigt!
end;


//*****************************************************************************
// Lade Spiele-Grafiken aus Datei und zeige erstes Bild zum Bearbeiten im
// Editor-Grid an. Knnen nicht genug / keine Bilder aus der Datei gelesen
// werden, erzeugt die Routine vier leere Standard-Grafiken
//
//  ->   FileName    Name der Datei aus der die Bilder geladen werden sollen
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.OpenGameBitmaps(FileName : TFileName);
var
 i       : Word;
 Size    : Byte;
 tmpBmp  : TBitmap;
begin
 g_GameBmp.AllFree;                                                     // Gebe vorige Bilderliste frei
 g_GameBmp := LoadGameBitmaps(FileName);                                // lade Bilder aus Datei
 if g_GameBmp = nil then                                                // wenns nicht geklappt hat
 begin
    ShowMessage(C_ERR_LOAD_BITMAPS+FileName+C_ERR_LOAD_DEFAULTS);       // Gebe Meldung aus
    g_GameBmp     := TExtImgLst.Create;                                 // Lege eigene Bilderliste an
    tmpBmp        := TBitmap.Create;                                    // erzeuge temp. Bild
    tmpBmp.Width  := 10;
    tmpBmp.Height := 10;
    tmpBmp.Canvas.Pen.Color   := clBlack;
    tmpBmp.Canvas.Brush.Color := clRed;
    tmpBmp.Canvas.Ellipse(tmpBmp.Canvas.ClipRect);
    for i:=1 to 4 do                                                    // fge temp. Bild vier mal in Bilderliste
       g_GameBmp.AddBmp(tmpBmp, 5);
    tmpBmp.Free;
 end;

 CurBmpNum := 1;                                                        // das erste Bild soll angezeigt werden
 g_BufferBmp.Assign( g_GameBmp.GetBmp(1,Size) );                        // Buffer bekommt erstes Bild!
 LoadBmpToGrid(g_BufferBmp, Size);                                      // und es wird angezeigt
 SpEdBmpNum.MaxValue := g_GameBmp.Count;                                // setze die Anzahl der Gegnerrauschiffe
 SpEdBmpNum.Value    := CurBmpNum;                                      // und das aktuelle Bild
end;


//*****************************************************************************
// Setzt den Pfad zur Exe-Datei Fighter.exe
//
//  ->   Path        Vollstndiger Pfad zur fighter.exe
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.SetGamePath(Path : String);
begin
 g_GamePath := Path;
end;


//*****************************************************************************
// Routine die das Drawgrid zum Bildeditor umbaut und das Zeichnen
// der einzelnen "Pixel" des Bildes im Grid bernimmt
//
// Wird fr jeden zu zeichnenden Kasten aufgerufen (Owner-Drawn-Control)
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.EditorGridDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
begin
 EditorGrid.Canvas.Brush.Color := g_BufferBmp.Canvas.Pixels[ACol, ARow];   // ermittel Farbe des Bild-Buffers an der Stelle
 EditorGrid.Canvas.FillRect(Rect);                                         // und zeichne Kasten in der Farbe
end;


//*****************************************************************************
// Die Bilderliste wird in der Default-Datei gespeichert
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.BtnSaveClick(Sender: TObject);
begin
 if MessageDlg(C_MSG_SAVE_BMPS, mtConfirmation, [mbYes, mbNo], 0) = mrYes then
 begin
    SaveCurImgLst(g_GamePath+C_GAME_BMP_FILE);                              // Bilderliste speichern
 end;
end;


//*****************************************************************************
// Spieler kann eine Datei aussuchen, in der die Bilderliste gespeichert wird
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.BtnExportClick(Sender: TObject);
begin
 if SaveDialog.Execute then                                             // wenn Dateiname ausgesucht
    SaveCurImgLst(SaveDialog.FileName);                                 // speicher die Datei
end;



//*****************************************************************************
// Die Gre des im Editor befindlichen Bildes wird gendert
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.GridMustChange(Sender: TObject);
begin
 if (Sender as TSpinEdit).Tag = 0 then                                  // kleiner Trick: Nur wenn Tag nicht gesetzt
 begin                                                                  // wird die nderung gemacht
    EditorGrid.DefaultColWidth      :=  SpEdZoom.Value;                 // (umgeht die autom. OnChange Aufruf-Problematik)
    EditorGrid.DefaultRowHeight     :=  SpEdZoom.Value;
    EditorGrid.RowCount             :=  SpEdHeight.Value;
    EditorGrid.ColCount             :=  SpEdWidth.Value;
    g_BufferBmp.Width               :=  SpEdWidth.Value;
    g_BufferBmp.Height              :=  SpEdHeight.Value;
 end;
end;


//*****************************************************************************
// Bietet die Mglichkeit ein Bitmap als Spielegrafik per Datei-Auswahl
// zu Importieren
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
// *** omportiere ein Bitmap als neue Grafik
procedure TEditorForm.BtnImportClick(Sender: TObject);
begin
 if OpenDialog.Execute then                                 // Der Spieler sucht Datei aus
    ImportBitmap(OpenDialog.FileName);                      // und das Bild wird importiert
end;


//*****************************************************************************
// Initialisiere die Bilderliste und den Buffer fr die aktuelle
// Bearbeitung
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.FormCreate(Sender: TObject);
begin
 g_BufferBmp   := TBitmap.Create;                           // Buffer fr aktuelles Bild
 g_GameBmp     := TExtImgLst.Create;                        // gesamte Bilderliste
 g_GameBmp.Init;
end;


//*****************************************************************************
// Gibt die Resourcen wieder frei
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.FormDestroy(Sender: TObject);
begin
 g_BufferBmp.Free;                                          // Buffer fr aktuelles Bild
 g_GameBmp.AllFree;                                         // gesamte Bilderliste
 g_GameBmp.Free;
end;


//*****************************************************************************
// Zeichnet ein Kstchen mit der aktuellen Farbe aus dem Color-Pick
//
// as, 26.09.2002  - Erste Version
//*****************************************************************************
// *** es wurde gezeichnet
procedure TEditorForm.EditorGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
 g_BufferBmp.Canvas.Pixels[ACol, ARow] := ColorGrid.ForegroundColor;    // Setzt Pixel im Buffer -> Mausklick lst Repaint aus
end;


//*****************************************************************************
// Anzeigen oder Ausblenden der Gitter-Linien im Editor
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.ChLinesClick(Sender: TObject);
begin
 EditorGrid.GridLineWidth  := Ord(ChLines.Checked);               // Liniendicke von: True = 1, False = 0
end;


//*****************************************************************************
// ndere den Zoom-Faktor des Editors
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.SpEdZoomChange(Sender: TObject);
begin
 EditorGrid.DefaultColWidth      :=  SpEdZoom.Value;              // setze Breite und Hhe des DrawGrids je nach Zoom
 EditorGrid.DefaultRowHeight     :=  SpEdZoom.Value;
end;


//*****************************************************************************
// Lade alle Spiel-Bilder und zeige das erste Bild zum Bearbeiten
// im Editor-Grid an
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.FormShow(Sender: TObject);
begin
 OpenGameBitmaps(g_GamePath+C_GAME_BMP_FILE);                      // lade Standard Bilder
end;


//*****************************************************************************
// Ein anderes Bild wurde ausgewhlt, die aktuellen nderungen am Bild
// im Editor werden gespeichert und das neue Bild wird geladen
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.BmpChange(Sender: TObject);
var
 Bmp  : TBitmap;
 Size : Byte;
begin
 g_GameBmp.ChangeBmp(CurBmpNum, g_BufferBmp, SpEdSize.Value);     // Speichere die Daten des aktuellen Bildes ab
 CurBmpNum  := SpEdBmpNum.Value;                                  // lade das neue, ausgewhlte Bild in den Editor
 Bmp        := g_GameBmp.GetBmp(CurBmpNum, Size);                 // Rckgabe ist ein Zeiger!
 LoadBmpToGrid(Bmp, Size);                                        // Zeige neues Bild im Editor
 BtnDelete.Enabled := (CurBmpNum > 3) AND (g_GameBmp.Count > 4);  // Gegner kann nur gelscht werden, wenn
end;                                                              // Gegner angezeigt und mind. 2 Gegner vorhanden


//*****************************************************************************
// Lade Spiel-Bilder aus Datei und zeige das erste Bild zum Bearbeiten
// im Editor-Grid an
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.BtnLoadClick(Sender: TObject);
begin
 if LoadDialog.Execute then
    OpenGameBitmaps(LoadDialog.FileName);                         // lade Grafiken aus ausgewhlter Datei
end;


//*****************************************************************************
// Fgt ein leeres Bild ans Ender der Bilder-Liste an
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.BtnAddClick(Sender: TObject);
begin
 AddEnemy;
end;


//*****************************************************************************
// Entfernt den aktuell ausgewhlten Gegner aus der Bilderliste
//
// as, 26.09.2002 - Erste Version
//*****************************************************************************
procedure TEditorForm.BtnDeleteClick(Sender: TObject);
begin
 DeleteEnemy(SpEdBmpNum.Value);                             // entferne ausgwhlten Gegner
end;

end.
