unit UMain;
//*****************************************************************************
//
// PROGRAMMNAME:        - FIGHTER alias "Space Invaders"
//                      - Dieses Programm stellt ein Computerspiel dar, bei
//                        dem der Spieler hnlich wie bei "Space Invaders" ein
//                        Raumschiff steuern kann und versuchen muss diverse
//                        Gegner abzuschieen ohne dabei selbst von den
//                        gegnerischen Schssen getroffen zu werden.
//                      - Das Spiel bietet auerdem einen Editor fr das
//                        Bearbeiten der Spielgrafiken, eine Highscore und
//                        verschiedene Optionen zum Spielverlauf
//
// Unit UMain           - Dieses ist die Hauptformular-Unit, die das Spiel-
// -----------            feld, die Spiel-Infos und das Hauptmen darstellt.
//                      - Sie enthlt Routinen zum Aufrufen der unterschiedl.
//                        Spielfunktionen wie Editor, Highscore etc. und
//                        Routinen die die Spiel-Ereignisse abfangen wie das
//                        Drcken der Steuertasten, ndern der Spielfeldgre
//                        oder die fr die Darstellung ntigen Timer-Ereignisse
//
// Stellt keine ffentliche Funktionen bereit!
// -------------------------------------------
//
//
// Verwendet eigene Units:
// -----------------------
// UOptions             - Enthlt Routinen zum Anzeige/Bearbeiten der Optionen
// UInput               - Steuert das Eingeben des Spielernamens fr die
//                        Highscore
// UObjFighter          - Enthlt die Basis-Klasse "TFighterObject" fr die
//                        allgmeinen Listen und Grafik-Objekt-Funktionen
// UType                - Enthlt wichtige Typen
// UObjPlayer           - enthlt alle Routinen zum Steuern und Anzeigen
//                        des Spieler-Raumschiffs und der Spieler-Schsse
// UObjEnemy            - Enthlt Routinen fr das Gegnerverhalten, insbesondere
//                        das Schussverhalten und Trefferverhalten
// UObjStar             - enthlt Routinen zum Zeichnen der Sterne
// UObjFire             - zeichnet Explosionen auf das Spielfeld
// UObjText             - zeigt Infos auf dem Spielfeld an
// UExtImgLst           - Enthlt Routinen fr eine Bilderliste
// UGame                - Enthlt allgemeine Routinen fr das Spiel
// UFile                - enthlt Routinen zum Lesen der Spiele-Grafiken aus
//                        verschiedenen Dateien
//
// Vordiploms-Programm  - (c) Januar 2003,  by Andreas Schibilla
// -------------------
//
//*****************************************************************************
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ExtCtrls, StdCtrls, Buttons, MMSystem, ImgList,
  UType, UObjPlayer, UObjEnemy, UObjStar, UObjFire, UObjText, UExtImglst;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    Game1: TMenuItem;
    MeExit: TMenuItem;
    MeOptions: TMenuItem;
    N1: TMenuItem;
    Image: TImage;
    MeEditor: TMenuItem;
    PaGameInfo: TPanel;
    ImLogo: TImage;
    LaLevelCaption: TLabel;
    LaLivesCaption: TLabel;
    LaPointsCaption: TLabel;
    LaPoints: TLabel;
    LaLevel: TLabel;
    LaLives: TLabel;
    LaTimeCaption: TLabel;
    LaLevelTime: TLabel;
    DrawTimer: TTimer;
    MeNeuesSpiel: TMenuItem;
    MePause: TMenuItem;
    N2: TMenuItem;
    LevelTimer: TTimer;
    MeHighscore: TMenuItem;
    EnemyTimer: TTimer;
    LaPause: TLabel;
    FireImageList: TImageList;
    procedure MeOptionsClick(Sender: TObject);
    procedure MeEditorClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure DrawTimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure MeNeuesSpielClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure MeExitClick(Sender: TObject);
    procedure LevelTimerTimer(Sender: TObject);
    procedure MeHighscoreClick(Sender: TObject);
    procedure MePauseClick(Sender: TObject);
    procedure EnemyTimerTimer(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ResizePlayfield(W,H : Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private-Deklarationen }
    g_BmpBuffer         : TBitmap;
    g_Stars             : TStars;
    g_Player            : TPlayer;
    g_Enemy             : TEnemy;
    g_Fire              : TFire;
    g_Text              : TText;
    g_Game              : TGame;
    g_OldStatus         : TGameStatus;
    g_LevelTime         : Cardinal;
    g_PlayerDead        : Cardinal;
    g_GameBitmaps       : TExtImgLst;
    g_EnemyTimerValue   : TEnemyShoot;
    procedure FinishedLevel;
    procedure NextLevel(Restart : Boolean);
    procedure GamePaused;
    procedure PlayerDead;
    procedure PlayerGameOver;
    procedure CheckGameStatus(var GameStatus : TGameStatus);
    procedure ShowHighscore;
  public
    { Public-Deklarationen }
  end;

var
  MainForm: TMainForm;

implementation

uses UEditor, UInput, UHighscore, UFile, UGame, UOptions;


{$R *.dfm}


//*****************************************************************************
// Ruft das Formular zum Bearbeiten der Optionen auf, wenn die Optionen
// fehlerfrei gelesen wurden konnten
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.MeOptionsClick(Sender: TObject);
var
 Game : TGame;
begin
 if LoadGameInfo(Game, ExtractFilePath(Application.EXEName)+C_CONFIG_FILE) then  // Wenn Laden der Optionen okay
 begin
    OptionsForm.SetGameInfo(Game);                                               // Setze Option im Formular
    if OptionsForm.ShowModal = mrOK then                                         // und zeige Formular an
       Beep;
 end
end;

//*****************************************************************************
// ffnet den Editor zum Bearbeiten der Spiele-Grafiken
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.MeEditorClick(Sender: TObject);
begin
 if EditorForm.ShowModal = mrOK then
    Beep;
end;


//*****************************************************************************
// ndert nach Grennderung des Formulars die entspr. Spiel-Objekte
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.FormResize(Sender: TObject);
begin
 ResizePlayfield(MainForm.ClientWidth-Image.Left-6, MainForm.ClientHeight-Image.Top-6);
end;


//*****************************************************************************
// Bewegt und zeichnet alle paar Milli-Sekunden alle Objekte auf dem
// Spielfeld
//
// ACHTUNG: sehr zeitkritische Funktion; Spielverlauf soll flssig bleiben
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.DrawTimerTimer(Sender: TObject);
begin
 g_BmpBuffer.Width               := Image.Width;            // Bereite Buffer vor
 g_BmpBuffer.Height              := Image.Height;
 g_BmpBuffer.Canvas.Brush.Color  := clBlack;                // Farbe ist schwarz
 g_BmpBuffer.Canvas.Pen.Color    := clSilver;               // und ein Silber-Rahmen
 g_BmpBuffer.Canvas.Rectangle(Image.Canvas.ClipRect);       // erstmal alles schwarz machen

 g_Stars.Draw(g_BmpBuffer.Canvas);                          // bewege und zeichne Sterne
 g_Enemy.Draw(g_BmpBuffer.Canvas, g_EnemyTimerValue);       // bewege und zeichne Gegner
 g_Enemy.DrawEnemyShoot(g_BmpBuffer.Canvas);                // bewege und zeichne Gegner-Schsse

 g_Player.DrawShoot(g_BmpBuffer.Canvas, g_Enemy, g_Text, g_Fire, g_Game.EnPoints);  // bewege und zeichne Spieler-Schsse
 if g_Player.Draw(g_BmpBuffer.Canvas, g_Game.Status, g_Enemy, g_Fire) then          // bewege und zeichne Spieler
    PlayerDead;                                                                     // bei Kollision -> Spieler getroffen

 g_Fire.Draw(g_BmpBuffer.Canvas, FireImageList, g_Game.NiceLook);                   // zeichne Explosionen
 g_Text.Draw(g_BmpBuffer.Canvas);                           // zeichne und dimme Text-Objekte

 CheckGameStatus(g_Game.Status);                            // berprfe den aktuellen Spielstatus

 Image.Picture.Bitmap := g_BmpBuffer;                       // Zeichenoperation erst jetzt sichtbar durchfhren!!!
 LaPoints.Caption := IntToStr(g_Player.GetPoints);          // und Spieler-Punkte setzen
end;


//*****************************************************************************
// Erzeugt alle fr das Spiel notwendigen Objekte und erzeugt einen doppelten
// Grafik-Buffer fr schnelleres Zeichnen
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.FormCreate(Sender: TObject);
begin
 // erzeuge Bitmap Buffer, jede Zeichenoperation geschieht zunchst
 // im Speicher auf dieser Struktur!
 g_BmpBuffer         := TBitmap.Create;

 // Delphi-Internes schnelles Grafik-Puffern aktivieren
 DoubleBuffered      := True;

 // Unterbinde alle Standard-Events des Spielfeldes -> schnellere Darstellung
 Image.ControlStyle  := Image.ControlStyle + [csNoStdEvents];

 // erzeuge alle fr das Spiel notwendigen Objekte
 g_Game.FieldSize    := Image.Width;
 g_Game.Status       := St_NoGame;
 g_Stars             := TStars.Create;
 g_Enemy             := TEnemy.Create;
 g_Text              := TText.Create;
 g_Fire              := TFire.Create;
 g_Player            := TPlayer.Create;
end;


//*****************************************************************************
// Startet ein neues Spiel. Alle alten Objekte, Listen etc. werden entfernt
// alle neuen erzeugt, alle Controls richtig gesetzt
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.MeNeuesSpielClick(Sender: TObject);
var
 W, H : Integer;
begin
 // Lade Spiele-Grafiken
 g_GameBitmaps := LoadGameBitmaps(ExtractFilePath(Application.EXEName)+C_GAME_BMP_FILE);
 if g_GameBitmaps = nil then
    ShowMessage(C_ERR_LOAD_DEF_BITMAPS);

 // Wenn Spiele-Grafiken und Spiele-Infos geladen
 if (g_GameBitmaps <> nil) and  LoadGameInfo(g_Game, ExtractFilePath(Application.EXEName)+C_CONFIG_FILE) then
 begin
    g_Game.Status       := St_FinishedLevel;          // setze Status
    g_Game.FieldSize    := Image.Width;               // setze Spielfeldgre
    LaPause.Visible     := False;                     // deaktiviere Pause-Label
    LaLives.Caption     := IntToStr(g_Game.Lives);    // zeige wieder volle Lebensanzahl an
    LaLevel.Caption     := '1';                       // erstes Level
    LaPoints.Caption    := '0';                       // Null Punkte
    // Falls vorhanden, entferne alte Objekt-Listen
    g_Stars.FreeLst;
    g_Text.FreeLst;
    g_Enemy.FreeLst;
    g_Enemy.FreeBmp;
    g_Player.FreeLst;
    g_Fire.FreeLst;
    // passe Imagegre an
    W := MainForm.ClientWidth - Image.Left - 6;
    H := MainForm.ClientHeight - Image.Top - 6;
    if H < W then
       W := H;
    Image.Width         := W;
    Image.Height        := W;
    // Initialisiere alle Spiele-Objekte
    g_Stars.Init(g_Game.StCount, g_Game.StMaxSpeed, Image.Width, Image.Height);
    g_Text.Init;
    g_Enemy.Init(Image.Width, g_GameBitmaps);
    g_Player.Init(Image.Width, Image.Height, g_GameBitmaps);
    // setze Status
    g_Game.Status       := St_Playing;
    g_LevelTime         := g_Game.LevelTime;
    // passe Objekte an Gre an
    g_Stars.Resize(C_DEF_PLAYFIELD,Image.Width);
    g_Enemy.Resize(C_DEF_PLAYFIELD,Image.Width);
    g_Player.Resize(C_DEF_PLAYFIELD,Image.Width);
    // Starte alle Timer
    DrawTimer.Enabled   := true;
    LevelTimer.Enabled  := true;
    EnemyTimer.Enabled  := true;
    g_PlayerDead        := 0;
    g_EnemyTimerValue   := 1;
 end;

end;


//*****************************************************************************
// Bewegt den Spieler nach links oder rechts oder ruft die Spielpause auf
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = g_Game.CtrRight then     // wenn Rechts-Taste gedrckt
     g_Player.GoLeft(True);         // bewege nach rechts
  if Key = g_game.CtrLeft then      // wenn Links-Taste gedrckt
     g_Player.GoRight(True);        // bewege nach links
  if Key = g_Game.CtrPause then     // wenn Pause gedrckt
     GamePaused;                    // pausiere das aktuelle Spiel
end;


//*****************************************************************************
// Stoppt die Bewegung des Spielers nach links oder rechts oder feuert einen
// Schu nach oben ab!
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = g_Game.CtrRight then                 // wenn Taste-Rechts losgelassen
     g_Player.GoLeft(False);                    // dann stoppe Bewegung nach rechts

  if Key = g_game.CtrLeft then                  // wenn Taste-Links losgelassen
     g_Player.GoRight(False);                   // dann stoppe Bewegung nach links

  if Key = g_Game.CtrFire then                  // wenn Feuertaste losgelassen
     g_Player.AddShoot(g_Game.Status);          // Setze Schu ab
end;


//*****************************************************************************
// Beendet das Spiel (Freigaben der Objekte, Listen etc. geschieht durch
// impliziten aufruf von FormDestroy)
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.MeExitClick(Sender: TObject);
begin
 Close;
end;


//*****************************************************************************
// Zhlt die Levelzeit und zeiht nach Ablauf der Zeit ein Leben ab
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.LevelTimerTimer(Sender: TObject);
begin
 if g_LevelTime > 1 then                                                      // wenn Levelzeit noch nicht abgelaufen
 begin
    Dec(g_LevelTime);                                                         // verringere um eine Sekunde
    LaLevelTime.Caption := SekToStr(g_LevelTime);                             // Zeige Zeit im Label an
    if g_LevelTime < 10 then                                                  // die letzten 10 Sekunden gro anzeigen
        g_Text.AddText(-1,-1, clYellow, 100, 0, 6, IntToStr(g_LevelTime));    // Text wird in Spielfeld gezeichnet
 end
 else
 begin                                                                        // Levelzeit abgelaufen
    LevelTimer.Enabled := False;                                              // Timer deaktivieren
    g_Text.AddText(-1,-1, clYellow, 24, 100, 1, C_MSG_TIME_OUT);              // Meldung ausgeben
    LaLevelTime.Caption := 'ABGELAUFEN';                                      // Label anpassen
    g_Enemy.FreeLst;                                                          // Gegner entfernen
    g_Game.Status := St_TimeOver;                                             // Status setzen
    PlayerDead;                                                               // Player ein Leben abziehen
 end;
end;


//*****************************************************************************
// Zeigt die Highscore an
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.MeHighscoreClick(Sender: TObject);
begin
 ShowHighscore;
end;


//*****************************************************************************
// Ruft die Spiel-Pause auf oder beendet sie
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.MePauseClick(Sender: TObject);
begin
 GamePaused;
end;


//*****************************************************************************
// Zhlt EnemyTimerValue hoch, damit die Gegner wissen, wann sie dran sind
// mit Schieen
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.EnemyTimerTimer(Sender: TObject);
begin
 if g_EnemyTimerValue < C_MAX_ENEMY_SHOOT then                             // Wenn noch nicht am Ende
    Inc(g_EnemyTimerValue)                                                 // erhhe Counter
 else
 begin
    g_EnemyTimerValue := 1;                                                // sonst Counter wieder auf 1
    g_Enemy.ShootUpdate(g_Game.EnFire + ( (g_Player.GetLevel-1) * g_Game.EnIncFire));  // und jeder Gegner kriegt neue Schsse
 end;
end;


//*****************************************************************************
// Setzt den aktuellen Pfad des Spiels fr die anderen Formulare
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.FormShow(Sender: TObject);
var
 GamePath : String;
begin
 GamePath := ExtractFilePath(Application.EXEName);    // ermittel Spiel-Pfad
 OptionsForm.SetGamePath(GamePath);                   // setze fr Options-Formular
 EditorForm.SetGamePath(GamePath);                    // setze fr Editor-Formular
end;


//*****************************************************************************
// Startet alle Objekte fr das Beenden des aktuellen Levels
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.FinishedLevel;
begin
 g_Enemy.FreeLst;       // lsche alle noch vorhandenen Text oder Gegner Objekte
 g_Text.FreeLst;
 if g_Player.GetLevel = 0 then                                                // das nchste Level ist das erste
    g_Text.AddText(-1,-1,clLime, 24, 30, 1, C_MSG_START_GAME)
 else
    g_Text.AddText(-1,-1,clLime, 24, 30, 1, C_MSG_LEVEL_FINISHED);            // normales neues Level

 LevelTimer.Enabled  := False;                                                // Levelzeit ist egal
 EnemyTimer.Enabled  := False;                                                // Gegner sollen nicht mehr schieen
 g_Game.Status       := St_FinishedLevel;                                     // setze Status
end;


//*****************************************************************************
// Startet alle Objekte fr das nchste Level
//
//  ->   Restart     soll das gleiche Level wie vorher neu gestartet werden?
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.NextLevel(Restart : Boolean);
begin
 if g_Text.NoText then
 begin
    if not Restart then                                  // wenn nicht gleiches Level gestartet werden soll
       g_Player.NextLevel;                               // gehe zum nchsten Level
    LaLevel.Caption := IntToStr(g_Player.GetLevel);      // und zeige das Level im Label an

    // berechne neue Geschwindigkeit fr Gegner!
    g_Enemy.SetEnemySpeed((g_Game.EnSpeed+( (g_Player.GetLevel-1) * g_Game.EnIncSpeed))*C_DEF_ENEMY_SPEED_POTI, Image.Width);
    // Fge Gegner fr neues Level hinzu
    g_Enemy.AddEnemies(Image.Width, Image.Height - g_Player.GetPlayerHeight - 40, g_Game.EnFire + ( (g_Player.GetLevel-1) * g_Game.EnIncFire),
                       g_Player.GetLevel, g_Game.EnCount, g_Game.EnIncCount);
    // Textausgabe, dass  nchstes Level gestartet wird
    g_Text.AddText(-1,300,clLime, 24, 100, 2, 'Start in Level '+IntToStr(g_Player.GetLevel)+' ...');

    g_Game.Status       := St_Playing;                   // setze Status
    g_LevelTime         := g_Game.LevelTime;             // und Levelzeit
    LevelTimer.Enabled  := True;                         // Timer wieder bereit
    LaLevelTime.Caption := SekToStr(g_Game.LevelTime);   // Label aktualisieren
    g_EnemyTimerValue   := 1;                            //
    EnemyTimer.Enabled  := True;                         // Gegner drfen wieder schieen
 end;
end;


//*****************************************************************************
// Pausiert ein Spiel oder beendet die Spiel-Pause
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.GamePaused;
begin
 if g_Game.Status = St_Pause then
 begin                                                // Pause beenden
    DrawTimer.Enabled   := DrawTimer.Tag =1;          // setze alle Timer zurck, wie sie vorher waren
    LevelTimer.Enabled  := LevelTimer.Tag=1;
    EnemyTimer.Enabled  := EnemyTimer.Tag=1;
    g_Game.Status       := g_OldStatus;
    LaPause.Visible     := False;                     // Pause-Label ausblenden
 end
 else
 if g_Game.Status <> St_NoGame then
 begin                                                // Spiel pausieren!
    DrawTimer.Tag       := Ord(DrawTimer.Enabled);    // merke, welchen Stand die Timer haben
    LevelTimer.Tag      := Ord(LevelTimer.Enabled);
    EnemyTimer.Tag      := Ord(EnemyTimer.Enabled);
    DrawTimer.Enabled   := False;                     // und dann deaktiviere alle Timer
    LevelTimer.Enabled  := False;
    EnemyTimer.Enabled  := False;
    g_OldStatus         := g_Game.Status;             // merke alten Spiel-Status
    g_Game.Status       := St_Pause;                  // setze neuen Spielstatus auf Pause
    LaPause.Caption     := '-PAUSE-';
    LaPause.Left        := Image.Left+1;              // Passe Pause-Label an das Spielfeld an
    LaPause.Top         := Image.Top+1;
    LaPause.Width       := Image.Width-2;
    LaPause.Height      := Image.Height-2;
    LaPause.Visible     := True;                      // und zeige das Pause-Label an!
 end;
end;


//*****************************************************************************
// berprft die Leben des Spielers, pat die Leben-Labels an oder zeigt
// GameOver
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.PlayerDead;
begin
 Dec(g_Game.Lives);                                                     // ein Spieler-Leben weniger
 if not PlaySound(C_SND_HITPLAYER, 0, SND_FILENAME or SND_ASYNC) then   // Explosions-Sound
    Beep;
 LaLives.Caption := IntToStr(g_Game.Lives);                             // Leben-Label anpassen
 if g_Game.Lives < 1 then                                               // ist der Spieler schon GameOver ?
    PlayerGameOver
 else
 begin
     if g_Game.Status <> St_TimeOver then
          g_Game.Status := St_PlayerDead;                               // ein Leben weniger
     EnemyTimer.Enabled := False;                                       // Gegner drfen jetzt kurz nicht schieen
     g_PlayerDead := C_PLAYERDEAD_TIME + C_PLAYER_COMEBACK_TIME;
 end;
end;


//*****************************************************************************
// berprft die Leben des Spielers, pat die Leben-Labels an oder zeigt
// GameOver
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.PlayerGameOver;
begin
 g_Game.Status       := St_NoGame;         // setze Spielstatus
 DrawTimer.Enabled   := False;             // alle Timer deaktivieren
 LevelTimer.Enabled  := False;
 EnemyTimer.Enabled  := False;

 LaPause.Caption     := '-GAME OVER-';     // Label fr Game-Over setzen
 LaPause.Left        := Image.Left+1;      // und die Gre an das Spielfeld anpassen
 LaPause.Top         := Image.Top+1;
 LaPause.Width       := Image.Width-2;
 LaPause.Height      := Image.Height-2;
 LaPause.Visible     := True;              // und Label sichtbar!

 // Player gut genug fr Highscore?
 if IsNewHighscore(ExtractFilePath(Application.EXEName)+C_CONFIG_FILE, g_Player.GetPoints) then
 begin
    if InputForm.ShowModal = mrOk then Beep; // Player soll seinen Namen eingeben
    // und der Eintrag wird zur Highscore hinzugefgt
    AddToHighscore(ExtractFilePath(Application.EXEName)+C_CONFIG_FILE, InputForm.GetInputString, g_Player.GetPoints);
    ShowHighscore;   // zeige Highscore
 end;

end;


//*****************************************************************************
// berprft den Spielstatus und setzt- rcksetzt dementsprechend alle
// Labels, Timer, Objekte etc.
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.CheckGameStatus(var GameStatus : TGameStatus);
begin
  case GameStatus of
    St_Playing:          if g_Enemy.NoEnemy then                           // wenn kein Gegner mehr da
                            FinishedLevel;                                 // beende dieses Level
    St_FinishedLevel:    NextLevel(False);                                 // Starte neues Level
    St_PlayerDead:       begin
                            EnemyTimer.Enabled := False;                   // Gegner drfen nicht mehr schieen
                            Dec(g_PlayerDead);
                            if g_PlayerDead = C_PLAYER_COMEBACK_TIME then  // warte mit dem neuerscheinen des Spielers
                               GameStatus := St_PlayerComeBack;
                         end;
    St_PlayerComeBack:   begin
                            Dec(g_PlayerDead);                             // Spieler erscheint, aber blinkt
                            if g_PlayerDead = 0 then
                            begin
                               GameStatus := St_Playing;                   // Spieler ist wieder normal
                               EnemyTimer.Enabled := True;                 // Gegner drfen wieder schieen
                            end;
                         end;
    St_TimeOver:         NextLevel(True);                                  // Zeit abgelaufen -> Level Restarten
 end;

end;


//*****************************************************************************
// Zeigt die Highscore an, wenn sie aus der Datei gelesen werden konnte
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.ShowHighscore;
var
 Highscore : THighscore;
begin
 // Wenn Highscore aus Datei geladen werden konnte,
 if LoadHighscore(Highscore,ExtractFilePath(Application.EXEName)+C_CONFIG_FILE) then
 begin
    HighscoreForm.SetHighscore(Highscore);                     // setze Highscore-Infos
    if HighscoreForm.ShowModal = mrOK then                     // zeige Highscore an
       Beep;
 end;
end;


//*****************************************************************************
// ndert nach Grennderung des Formulars die entspr. Spiel-Objekte
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.ResizePlayfield(W, H : Integer);
begin
 if DrawTimer.Enabled then
 begin
    DrawTimer.Enabled   := False;                                    // deaktiviere Timer
    if H < W then                                                    // der kleinere Wert ist bindend
       W := H;
    Image.Width         := W;                                        // setze Spielfeld
    Image.Height        := W;
    g_Player.Resize(g_Game.FieldSize, W);                            // passe Player-Objekt an
    g_Stars.Resize(g_Game.FieldSize, W);                             // passe Sterne-Objekt an
    g_Enemy.Resize(g_Game.FieldSize, W);                             // passe Gegner-Objekt an
    g_Game.FieldSize    := Image.Width;
    DrawTimer.Enabled   := True;                                     // reaktiviere Timer
 end;

end;


//*****************************************************************************
// Deaktiviert alle Timer, entfernt alle Objekte und gibt den belegten
// Speicher wieder frei
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 DrawTimer.Enabled  := False; // deaktiviere alle Timer
 LevelTimer.Enabled := False;
 EnemyTimer.Enabled := False;
 g_BmpBuffer.Free;            // gebe Double-Buffer wieder frei

 g_Stars.FreeLst;             // gebe Sterne-Objekt wieder frei
 g_Stars.Free;

 g_Player.FreeLst;            // gebe Spieler-Objekt wieder frei
 g_Player.Free;

 g_Enemy.FreeLst;             // gebe Gegner-Objekt wieder frei
 g_Enemy.Free;

 g_Fire.FreeLst;              // gebe Explosions-Objekt wieder frei
 g_Fire.Free;

 g_Text.FreeLst;              // gebe Text-Objekt wieder frei
 g_Text.Free;

 // Zum Debuggen der Zeigerfunktionen
 // ShowMessage('PtrDebug: '+IntToStr(PtrDebug)+#13+'LstDebug: '+IntToStr(PtrDebug) );

end;

initialization
Randomize;
//PtrDebug := 0;        // Zum Debuggen der Zeigerfunktionen
//LstDebug := 0;        // Zum Debuggen der Zeigerfunktionen


end.
