unit UObjFighter;
//*****************************************************************************
// Unit UObjFighter     - Diese Unit enthlt die Klasse TFighterObject inkl.
// ----------------       aller Deklarationen und Member-Methoden
//                      - Die Klasse TFighterObject ist die Basis-Klasse fr
//                        die anderen Fighter-Klassen. Sie stellt Routinen
//                        zur Listen-Bearbeitung, Abdunklung, Positions-
//                        berechnung und Bildgrenanpassung zur Verfgung!
//
// ffentliche-Methoden:
// ---------------------
// Init                 - Initialsiert eine Basis-Liste
// Add                  - Fgt ein neues Daten-Element am Anfang der Liste ein
// AllFree              - Entfernt alle Elemtne und gibt belegten Speicher frei
// FreePtr              - Entfernt ein beliebiges Element aus der Liste
// Darken               - Dunkelt einen Farbwert ab -> bis ins Schwarze
// IsCollision          - Prft, ob sich zwei Rechtecke berlappen
// PosChange            - berechnet neue Position nach Spielfeldgrennderung
// ResizeBmp            - Passt eine Spiel-Grafik an Spielfeldgrennderung an
//
// Verwendet eigene Units:
// -----------------------
// UType                - Enthlt wichtige Typen
//
// Vordiploms-Programm  - (c) Januar 2003,  by Andreas Schibilla
// -------------------
//
//*****************************************************************************
interface

uses Windows, Graphics, UType;

type
 TFighterObject = class
                        procedure Init(var ListPtr : TFighterPtr);
                        procedure Add(var ListPtr : TFighterPtr; Data : TFighterRec);
                        procedure AllFree(var ListPtr : TFighterPtr);
                        procedure FreePtr(var ListPtr : TFighterPtr; var Ptr : TFighterPtr);
                        function  Darken(var Col : TColor; Speed : Byte) : Boolean;
                        function  IsCollision(Rec1, Rec2 : TRect) : Boolean;
                        function  PosChange(OldField, NewField, Pos :  Integer) : Integer;
                        procedure ResizeBmp(Bmp, BmpOrig : TBitmap; Playfield : Integer; Percent : Byte);
                  end;


implementation

//*****************************************************************************
// Initialisiert eine Fighter-Liste
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TFighterObject.Init(var ListPtr : TFighterPtr);
begin
 ListPtr := nil;
end;


//*****************************************************************************
// Fgt ein neues Daten-Element am Anfang der Liste ein
//
// <->   ListPtr     Zeiger auf den Anfang der Liste
//  ->   Data        Datenblock, der als Element eingfgt werden soll
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TFighterObject.Add(var ListPtr : TFighterPtr; Data : TFighterRec);
var
   tmpPtr : TFighterPtr;
begin
 New(tmpPtr);                       // erzegt Speicher fr ein neues Element
 // Inc(PtrDebug);                     // zum Debuggen der Zeigerfunktionen
 tmpPtr^.Data  := Data;             // Element bekommt die richtigen Daten
 tmpPtr^.Next  := ListPtr;          // Neues Element zeigt auf altes erstes Element
 tmpPtr^.Prev  := nil;              // Vorgnger gibt es nicht

 if ListPtr <> nil then             // Wenn schon Elemente in der Liste
    ListPtr^.Prev := tmpPtr;        // altes erstes El. zeigt auf neues erstes Element

 ListPtr       := tmpPtr;           // Anfang der Liste ist jetzt das neue Element
end;


//*****************************************************************************
// Entfernt alle Elemtne und gibt den belegten Speicher der Liste wieder frei
//
// <->   ListPtr     Zeiger auf den Anfang der Liste
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TFighterObject.AllFree(var ListPtr : TFighterPtr);
var
 tmpPtr : TFighterPtr;
begin
 tmpPtr := ListPtr;                 // gehe zum Anfang der Liste
 while tmpPtr <> nil do             // und durchlaufe jedes Element bis zum Ende
 begin
    ListPtr  := tmpPtr^.Next;       // Anfang ist nchstes Element
    Dispose(tmpPtr);                // gebe Element frei
    // Dec(PtrDebug);                  // zum Debuggen der Zeigerfunktionen
    tmpPtr   := ListPtr;            // und ab zum nchsten Element
 end;
 ListPtr := nil;                    // Liste ist leer!
end;


//*****************************************************************************
// Entfernt ein beliebiges Element aus der Liste
//
// <->   ListPtr     Zeiger auf den Anfang der Liste
// <->   Ptr         Zeiger auf das Element der Liste, das Entfernt werden soll
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TFighterObject.FreePtr(var ListPtr : TFighterPtr; var Ptr : TFighterPtr);
begin
 if Ptr <> nil then                          // wenn gltiger Zeiger bergeben
 begin
      if Ptr^.Prev <> nil then               // wenn Vorgnger vorhanden
           Ptr^.Prev^.Next := Ptr^.Next      // Vorgnger zeigt auf Nachfolger
      else
           ListPtr := Ptr^.Next;             // sonst, erstes Element zeigt auf Nachfolger

      if Ptr^.Next <> nil then               // Wenn Nachfolger vorhanden
           Ptr^.Next^.Prev := Ptr^.Prev;     // Nachfolger zeigt auf Vorgnger

      Dispose(Ptr);                          // gebe Element frei
      // Dec(PtrDebug);                         Zum Debuggen der Zeigerfunktionen
      Ptr := nil;                            // alter Zeiger muss dann auf nil zeigen
 end;
end;


//*****************************************************************************
// Dunkelt einen Farbwert ab -> bis ins Schwarze
//
// <->   Col         Farbwert der etwas abgedunkelt wird
//  ->   Speed       Wie stark/schnell soll der Farbwert ins schwarze bergehen
// <                 Gibt zurck, ob Farbwert nach Abdunkeln exakt schwarz ist
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
function TFighterObject.Darken(var Col : TColor; Speed : Byte) : Boolean;
var R, G, B: Byte;
begin
 R := GetRValue(Col);                              // ermittel Rot-Anteil der Farbe
 G := GetGValue(Col);                              // ermittel Grn-Anteil der Farbe
 B := GetBValue(Col);                              // ermittel Blau-Anteil der Farbe

 if R >= Speed then Dec(R, Speed) else R := 0;     // wenn mglich, verringer Rot-Anteil
 if G >= Speed then Dec(G, Speed) else G := 0;     // wenn mglich, verringer Grn-Anteil
 if B >= Speed then Dec(B, Speed) else B := 0;     // wenn mglich, verringer Blau-Anteil

 Col := RGB(R, G, B);                              // abgedunkelte Farbe wird aus den verringerten Anteilen berechnet
 Darken := ((R=0) and (G=0)) and (B=0);            // gebe zurck, ob neue Farbe exakt schwarz
end;


//*****************************************************************************
// Prft, ob sich zwei Rechtecke berlappen (kollidieren)
//
//  ->   Rec1        Rechteck 1
//  ->   Rec2        Rechteck 2
// <                 Gibt zurck, ob sich die Rechtecke berlappen
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
function TFighterObject.IsCollision(Rec1, Rec2 : TRect) : Boolean;
begin
IsCollision := not   (
 (Rec1.Left   > Rec2.Right) or        // links neben
 (Rec1.Right  < Rec2.Left)  or        // rechts neben
 (Rec1.Bottom < Rec2.Top)   or        // unter
 (Rec1.Top    > Rec2.Bottom)          // ber
                     )
end;


//*****************************************************************************
// Rechnet eine Position auf dem Spielfeld nach einer Spielfeldgrennderung
// um in die entsprechende neue Position
//
//  ->   OldField    alte Spielfeldgre
//  ->   NewField    neue Spielfeldgre
// <                 Gibt die neue Position zurck
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
function TFighterObject.PosChange(OldField, NewField, Pos :  Integer) : Integer;
begin
 if OldField > 0 then
   PosChange := Round((Pos * NewField) / OldField)    // Prozentual muss Position gleich bleiben
 else
   PosChange := Pos;
end;


//*****************************************************************************
// Passt eine Spiel-Grafik an einen Spielfeldgrennderung an
//
//  ->   Bmp         zu nderndes Bild
//  ->   BmpOrig     Original-Bild
//  ->   Playfield   neue Spielfeldgre
//  ->   Percent     Gre (in %) des Bildes auf dem Spielfeld
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TFighterObject.ResizeBmp(Bmp, BmpOrig : TBitmap; Playfield : Integer; Percent : Byte);
begin
 Bmp.Width  := Round ( (Playfield / 100) * Percent );                   // berechne neue Breite fr Spielfigur
 Bmp.Height := Round ( Bmp.Width / BmpOrig.Width * BmpOrig.Height );    // berechne die dazu passende, neue Hhe

// Bmp.Canvas.Brush.Color := clBlack;
// Bmp.Canvas.FillRect(Bmp.Canvas.ClipRect);
 Bmp.Canvas.StretchDraw(Bmp.Canvas.ClipRect, BmpOrig);                  // Stretche das Original in die neue Gre
end;



end.
