unit UObjPlayer;
//*****************************************************************************
// Unit UObjPlayer      - Diese Unit enthlt die Klasse TPlayer inklusive
// ---------------        aller Deklarationen und Member-Methoden
//                      - Die Klasse TPlayer dient zur Anzeige und Bewegung
//                        nach links/rechts des Spielerraumschiffs
//                      - auerdem steuert sie die Spieler-Schsse/Kollisionen
//
// ffentliche-Methoden:
// ---------------------
// Init                 - Initialsiert Spieler- und Schugrafik, Position,
//                        Level, Punktzahl, Spieler-Schsse etc.
// GoLeft               - Setzt Bewegung des Spielers nach links in Gang
// GoRight              - Setzt Bewegung des Spielers nach rechts in Gang
// MovePlayer           - Bewegt das Spieler-Raumschiff auf dem Spielfeld
// Draw                 - Bewegt und zeichnet das Spieler-Raumschiff
//                        Prft anschlieend, ob Spieler getroffen wurde
// DrawShoot            - Bewegt und zeichnet die Spieler-Schsse, prft
//                        anschlieend, ob Schu einen Gegner getroffen hat
// AddShoot             - Fgt einen neuen Schu in die Schu-Liste
// Resize               - Pat die Gren, Positionen und Geschw. des
//                        Spieler-Raumschiffs/Schsse an neue Spielfeldgre an
// NextLevel            - Der Spieler geht ins nchste Level
// GetPoints            - Gibt die aktuelle Spieler-Punktzahl zurck
// GetLevel             - Gibt das aktuelle Spieler-Level zurck
// GetPlayerHeight      - Gibt die Hhe des Spieler-Raumschiffes zurck
// GetPlayerWidth       - Gibt die Breite des Spieler-Raumschiffes zurck
// FreeLst              - Entfernt alle Spieler-Schsse und Spieler/Schu-
//                        Grafiken und gibt den belegten Speicher wieder frei
//
// Verwendet eigene Units:
// -----------------------
// UType                - Enthlt wichtige Typen
// UFile                - enthlt Routinen zum Lesen der Spiele-Grafiken aus
//                        verschiedenen Dateien
// UObjFighter          - Enthlt die Elternklasse "TFighterObject", von der
//                        sich diese Klasse ableitet
// UObjEnemy            - Enthlt Routinen fr das Gegnerverhalten, insbesondere
//                        das Schussverhalten und Trefferverhalten
// UObjText             - zeigt Infos auf dem Spielfeld an
// UObjFire             - zeichnet Explosionen auf das Spielfeld
// UExtImgLst           - Stellt Routinen fr Bilderlisten zur Verfgung
//
// Vordiploms-Programm  - (c) Januar 2003,  by Andreas Schibilla
// -------------------
//
//*****************************************************************************
interface

uses Types, MMSystem, Graphics, SysUtils, UType, UObjFighter, UObjEnemy, UObjText, UObjFire, UExtImgLst;

type
TPlayer = class(TFighterObject)
                private
                ShootCount      : Integer;      // Wieviele Spieler-Schsse sind auf dem Spielfeld
                Left, Right     : Boolean;      // In welche Richtung bewegt sich der Spieler
                x, y            : integer;      // Wo befindet sich der Spieler
                Speed           : Integer;      // aktuelle Geschw. des Spielers
                ShootSpeed      : Integer;      // Wie schnell ist der Spieler-Schu
                ListPtr         : TFighterPtr;  // Zeiger auf Anfang der Liste mit den Spieler-Schssen
                Points          : Cardinal;     // Punktzahl des Spielers
                Level           : Integer;      // akutelles Level

                PlayerBmpOrig   : TBitmap;      // Spieler-Raumschiff Original-Bild
                PlayerBmp       : TBitmap;      // das Spieler-Raumschiff-Bild an die aktuelle Spielfeldgre angepat
                PlayerBmpSize   : Byte;         // Prozent des Spieler-Raumschiffs auf dem Spielfeld
                ShootBmpOrig    : TBitmap;      // Spieler-Schu Original-Bild
                ShootBmp        : TBitmap;      // das Spieler-Schu-Bild an die aktuelle Spielfeldgre angepat
                ShootBmpSize    : Byte;         // Prozent des Spieler-Schusses auf dem Spielfeld
                public
                procedure Init(Max_x, Max_y : integer; GameBitmaps : TExtImgLst);
                procedure GoLeft(flag : Boolean);
                procedure GoRight(flag : Boolean);
                procedure MovePlayer(PlayField : Integer);
                function  Draw(Canvas : TCanvas; Status : TGameStatus; var Enemy : TEnemy; var Fire : TFire) : Boolean;
                procedure DrawShoot(Canvas : TCanvas; var Enemy : TEnemy; var Text : TText; var Fire : TFire; KillPoints : Integer);
                procedure AddShoot(GameStatus : TGameStatus);
                procedure Resize(OldField, NewField : Integer);
                procedure NextLevel;
                function  GetPoints : Cardinal;
                function  GetLevel : Integer;
                function  GetPlayerHeight : Integer;
                function  GetPlayerWidth  : Integer;
                procedure FreeLst;
          end;

implementation

uses UFile;

//*****************************************************************************
// Initialisiert die Spieler- und Schugrafiken, initialisiert die Schu-
// liste, rcksetzt Punkte, Level, Position etc.
//
//  ->   Max_x       Spielfeldbreite
//  ->   Max_y       Spielfeldhhe
//  ->   GameBitmaps Bilderliste, die Spiele-Grafiken enthlt
//                   (1. Bild ist Spieler-Raumschiff, 2. Bild ist Spieler-Schu)
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
procedure TPlayer.Init(Max_x, Max_y : integer; GameBitmaps : TExtImgLst);
var
 tmpBmp : TBitmap;
begin
 ListPtr       := nil;                                         // Liste ist noch leer
 ShootCount    := 0;                                           // noch kein Schuss abgefeuert

 PlayerBmpOrig := TBitmap.Create;                              // erzeuge alle ntigen Bilder
 PlayerBmp     := TBitmap.Create;
 ShootBmpOrig  := TBitmap.Create;
 ShootBmp      := TBitmap.Create;
 tmpBmp        := TBitmap.Create;

 tmpBmp.Assign( GameBitmaps.GetBmp(1, PlayerBmpSize) );        // ermittel Spieler-Grafik
 PlayerBmpOrig.Assign(tmpBmp);                                 // speicher das Bild in Originalgre
 PlayerBmpOrig.TransparentColor  := clBlack;
 PlayerBmpOrig.Transparent       := C_TRANSPARENT;
 PlayerBmp.Assign(tmpBmp);                                     // und speicher das Bild fr angepate Gre
 PlayerBmp.TransparentColor      := clBlack;
 PlayerBmp.Transparent           := C_TRANSPARENT;

 tmpBmp.Assign( GameBitmaps.GetBmp(2, ShootBmpSize) );         // ermittel Spieler-Schu-Grafik
 ShootBmpOrig.Assign(tmpBmp);                                  // speicher das Bild in Originalgre
 ShootBmpOrig.TransparentColor   := clBlack;
 ShootBmpOrig.Transparent        := C_TRANSPARENT;
 ShootBmp.Assign(tmpBmp);                                      // und speicher das Bild gr angepate Gre
 ShootBmp.TransparentColor       := clBlack;
 ShootBmp.Transparent            := C_TRANSPARENT;
 tmpBmp.Free;                                                  // gebe Temp-Bild frei

 x          := (Max_x div 2) - (PlayerBmp.Width div 2);        // berechne X-Position => Spielfeldmitte
 y          := Max_y - PlayerBmp.Height - 1;                   // berechne Y-Position => unten am Rand
 Speed      := C_PLAYER_MOVE_SPEED;                            // Wie schnell kann sich Spieler bewegen
 ShootSpeed := C_PLAYER_SHOOT_SPEED;                           // und wie schnell die abgefeuerten Schsse
 Points     := 0;                                              // Punkte zurcksetzen
 Level      := 0;                                              // beginne im ersten Level (wird am Anfang hochgezhlt)
 Left       := False;                                          // noch nicht nach links bewegen
 Right      := False;                                          // noch nicht nach rechts bewegen
end;


//*****************************************************************************
// Setzt die Bewegung des Spieler-Raumschiffes nach links in Gang oder
// stoppt sie
//
//  ->   flag        Soll nach links bewegt werden?
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
procedure TPlayer.GoLeft(flag : Boolean);
begin
 Left  := flag;
end;


//*****************************************************************************
// Setzt die Bewegung des Spieler-Raumschiffes nach rechts in Gang oder
// stoppt sie
//
//  ->   flag        Soll nach rechts bewegt werden?
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
procedure TPlayer.GoRight(flag : Boolean);
begin
 Right := flag;
end;


//*****************************************************************************
// Bewegt das Spieler-Raumschiff auf dem Spielfeld
//
//  ->   PlayField   Gre des Spielfelds
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
procedure TPlayer.MovePlayer(PlayField : Integer);
begin
 if Left then                                      // nach links bewegen?
        Inc(x, Speed)
 else if Right then                                // nach rechts bewegen?
        Dec(x, Speed);
 if x < 1 then                                     // aufpassen, dass Raumschiff nicht links aus Spielfeld bewegt
        x := 1
 else if (x+PlayerBmp.Width) > PlayField then      // aufpassen, dass Raumschiff nicht rechts aus Spielfeld bewegt
        x := PlayField - PlayerBmp.Width;
end;


//*****************************************************************************
// Bewegt und zeichnet das Spieler-Raumschiff
// Prft anschieend, ob Spieler durch Schu oder Gegnerkollision getroffen
// wurde
//
//  ->   Canvas      Zeichenflche, auf der gezeichnet werden soll
//  ->   Status      akuteller Spielstatus
// <->   Enemy       zu verwendene Gegner-Objekt
// <->   Fire        zu verwendene Explosions-Objekt
// <                 Gibt zurck, ob der Player nach Bewegung getroffen wurde
//                   (entweder durch Gegner-Schuss oder Gegner-Kollision)
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
function TPlayer.Draw(Canvas : TCanvas; Status : TGameStatus; var Enemy : TEnemy; var Fire : TFire) : Boolean;
var
   Rec1             : TRect;
begin
 Draw := False;                                                      // Gegner wurde noch nicht getroffen
 case Status of
 St_Playing:         begin                                           // normaler Spiel-Status
                     MovePlayer(Canvas.ClipRect.Right-1);            // bewege ggf. das Spieler-Raumschiff
                     Canvas.Draw(x, y, PlayerBmp);                   // zeichne es dann auf die Zeichenflche
                     Rec1.Left     := x;                             // erstelle Rechteckstruktur aus Spielerfigur
                     Rec1.Top      := y;
                     Rec1.Right    := x + PlayerBmp.Width;
                     Rec1.Bottom   := y + PlayerBmp.Height;
                     if Enemy.HitEnemy(Rec1, Fire) or                // Prfe, ob Kollision mit Gegner?
                        Enemy.HitEnemyShoot(Rec1) then               // oder von Gegner abgeschossen?
                        begin
                           Fire.AddFire(Rec1.Left,Rec1.Top,Rec1.Right,Rec1.Bottom,C_FIRE_SPEED);   // Explosion zeichnen
                           Draw := True;                             // Gegner wurde getroffen!!!
                        end;
                     end;
 St_PlayerDead:      Beep;                                           // Wenn Spieler tot, kurze Bieptne
 St_PlayerComeBack:  begin                                           // Spieler erscheint wieder
                     MovePlayer(Canvas.ClipRect.Right-1);            // bewege ihn
                     if Random(2) = 0 then                           // zeichne ihn nicht jedesmal, nur ca. jedes 2x
                        Canvas.Draw(x, y, PlayerBmp);                // das ergibt Blink-Effekt!
                     end;
 else                begin                                           // bewege und zeichne Spieler ganz normal
                     MovePlayer(Canvas.ClipRect.Right-1);            // z.B. wenn Level geschafft...
                     Canvas.Draw(x, y, PlayerBmp);
                     end;
 end;

end;


//*****************************************************************************
// Bewegt und zeichnet die Spieler-Schsse, berprft anschlieend, ob ein
// Schu einen Gegner getroffen hat
//
//  ->   Canvas      Zeichenflche auf der gezeichnet wird
// <->   Enemy       Gegner-Objekt zur berprfung auf Treffer
// <->   Text        Text-Objekt zum Zeichnen der Punkte bei Treffer
// <->   Fire        Explosions-Objekt zum Zeichnen der Explosion
//  ->   KillPoints  Anzahl der Punkte, die der Spieler fr Gegner bekommt
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
procedure TPlayer.DrawShoot(Canvas : TCanvas; var Enemy : TEnemy; var Text : TText; var Fire : TFire; KillPoints : Integer);
var
   tmpPtr, delPtr   : TFighterPtr;
   Rec1             : TRect;
   dummy            : Boolean;
begin
 tmpPtr := ListPtr;
 while tmpPtr <> nil do                                     // durchlaufe jeden abgefuerten Schuss
 begin
   with tmpPtr^.Data do                                     // Daten zum aktuellen Schuss
   begin
      Dec(y, ShootSpeed);                                   // bewege Schuss je nach Geschwindigkeit
      delPtr := tmpPtr;                                     // merke Zeiger des Schusses
      if y >= 1 then                                        // wenn Schuss noch auf Spielfeld
      begin
         Canvas.Draw(x, y, ShootBmp);                       // zeichne Schuss
         Rec1.Left   := x;                                  // ermittel Rechteck des Schusses
         Rec1.Top    := y;
         Rec1.Right  := x+ShootBmp.Width;
         Rec1.Bottom := y+ShootBmp.Height;
         tmpPtr := tmpPtr^.next;
         if Enemy.HitEnemy(Rec1, Fire) then                 // Gucke, ob Schuss einen Gegner getroffen hat
         begin
                if not PlaySound(C_SND_HITENEMY, 0, SND_FILENAME or SND_ASYNC or SND_NOWAIT) then // Getroffen -> Sound!
                   Beep;
                Text.AddText(x,y,C_KILLPOINTSCOLOR, 14, 10, 5, IntToStr(KillPoints)+' Punkte'); // und Trefferpunkte ausgeben
                Inc(Points, KillPoints);                                                        // Spieler bekommt Punkte
                FreePtr(ListPtr, delPtr);                                                       // Schu muss weg
                Dec(ShootCount);
         end;
      end
      else                                                  // Schuss ist aus dem Spielfeld oben raus
      begin
         tmpPtr := tmpPtr^.next;                            // gehe zum nchsten Schuss
         FreePtr(ListPtr, delPtr);                          // aber entferne den aktuellen Schuss aus der Liste
         Dec(ShootCount);                                   // und passe die Anzahl der Schss an
      end;

   end;
 end;

end;


//*****************************************************************************
// Fgt einen neuen Schu in die Schu-Liste
//
//  ->   GameStatus  aktueller Spiel-Status
// <                 Gibt zurck ob Hinzufgen des Schusses geklappt hat
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
procedure TPlayer.AddShoot(GameStatus : TGameStatus);
var
 Data  : TFighterRec;
begin
 if (GameStatus = St_Playing)    and                                    // wenn ein Spiel aktiv und
    (ShootCount < C_MAX_PLAYER_SHOOT) then                              // noch nicht max. Anzahl Schsse auf Spielfeld
 begin
    Data.x     := x + (PlayerBmp.Width div 2) - (ShootBmp.Width div 2); // Schu-Position ist Mitte des Spieler-Raumschiffs
    Data.y     := y;                                                    // und gleiche y-Position
    Add(ListPtr, Data);                                                 // fge Schu hinzu
    Inc(ShootCount);                                                    // erhhe Schu-Zhler
    if not PlaySound(C_SND_PLAYERFIRE, 0, SND_FILENAME or SND_ASYNC or SND_NOWAIT) then
       Beep;
 end
end;


//*****************************************************************************
// Pat die Gren, Positionen und Geschwindigkeiten des Spieler-Raumschiffs
// und der Spieler-Schsse an neue Spielfeldgre an
//
//  ->   OldField    Alte Spielfeldgre
//  ->   NewField    Neue Spielfeldgre
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
procedure TPlayer.Resize(OldField, NewField : Integer);
begin
 ResizeBmp(PlayerBmp, PlayerBmpOrig, NewField, PlayerBmpSize);    // passe Spieler-Bitmap an die neue Gre an
 ResizeBmp(ShootBmp, ShootBmpOrig, NewField, ShootBmpSize);       // passe Spieler-Schuss-Bitmap an die neue Gre an

 x := PosChange(OldField, NewField, x);                           // Passe Spieler-Position an die neue Gre an
 y := NewField - PlayerBmp.Height - 1;

 // Passe Geschwindigkeit an die neue Gre an
 if OldField <> 0 then
   begin
   Speed       := Round ( (NewField / OldField) * Speed );
   ShootSpeed  := Round ( (NewField / OldField) * ShootSpeed );
   end;
 if Speed < 1 then                                                // keine ungltigen Werte fr Speed
   Speed := 1;
 if ShootSpeed < 1 then                                           // keine ungltigen Werte fr ShhotSpeed
   ShootSpeed := 1;
   
end;


//*****************************************************************************
// Der Spieler geht ins nchste Level.
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
procedure TPlayer.NextLevel;
begin
 Inc(Level);
end;


//*****************************************************************************
// <                 Gibt die aktuelle Spieler-Punktzahl zurck
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
function TPlayer.GetPoints : Cardinal;
begin
 GetPoints := Points;
end;


//*****************************************************************************
// <                 Gibt das aktuelle Spieler-Level zurck
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
function TPlayer.GetLevel : Integer;
begin
 GetLevel := Level;
end;


//*****************************************************************************
// <                 Gibt die aktuelle Hhe des Spieler-Raumschiffes zurck
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
function TPlayer.GetPlayerHeight : Integer;
begin
 GetPlayerHeight := PlayerBmp.Height;
end;


//*****************************************************************************
// <                 Gibt die aktelle Breite des Spieler-Raumschiffes zurck
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
function TPlayer.GetPlayerWidth : Integer;
begin
 GetPlayerWidth := PlayerBmp.Width;
end;

//*****************************************************************************
// Entfernt alle Spieler-Schsse und Spieler/Schu-Grafiken und gibt den
// belegten Speicher wieder frei
//
// as, 22.09.2002 - Erste Version
//*****************************************************************************
procedure TPlayer.FreeLst;
var
 tmpPtr : TFighterPtr;
begin
 tmpPtr := ListPtr;              // gehe zum Anfang der Spieler-Schu-Liste
 while tmpPtr <> nil do          // durchlaufe jeden Schuss
   begin
   ListPtr  := tmpPtr^.Next;     // neuer Anfang ist nchstes Element
   Dispose(tmpPtr);              // Gebe aktuelles Element frei!
   tmpPtr   := ListPtr;          // und ab zum nchsten Element
   end;
 ListPtr := nil;                 // Liste ist wieder leer
 PlayerBmp.Free;                 // Gebe Grafiken frei
 PlayerBmpOrig.Free;
 ShootBmp.Free;
 ShootBmpOrig.Free;
end;


end.
